import os
from glob import glob

# import all extensions
for module in os.listdir(os.path.dirname(__file__)):
    if module == '__init__.py' or module[-3:] != '.py':
        continue
    try:
        __import__(module[:-3], locals(), globals())
    except Exception as e:
        print('Exception {} in extension {}'.format(e, module))
del module



from datetime import datetime
import sys
import os

from bwikibot.api import Wiki, datetime2zulu
from bwikibot.ui import shift_block

SESSION_FILE = 'cli.session'
SECONDARY_SESSION_FILE = 'cli_secondary.session'

def main():
    run(*sys.argv)

def run(*argv):
    load_extensions()
    if len(argv) < 2:
        print('You should pass action:')
        show_doc(actions)
    else:
        action = actions.get(argv[1])
        if action:
            action(*argv[2:])
        else:
            print('Action "%s" not found.' % argv[1])
            print('Available actions: %s' % ', '.join(actions.keys()))
    
actions = {}

def action(name):
    ''' Decorator to register function as action '''
    def decorate(f):
        actions[name] = f
        return f
    return decorate

def load_extensions():
    ''' Trigger action decorator in extensions '''
    from bwikibot import extensions

def show_doc(actions):
    ''' Prints documentation about actions '''
    maxname = max(len(k) for k in actions.keys())
    for action, function in actions.items():
        doc = function.__doc__ or ''.join((
            function.__module__, '.',
            function.__name__, ' has no docstring'
        ))
        print('{:{width}s} - {}'.format(
            action,
            shift_block(doc, maxname + 5),
            width=maxname,
        ))

def get_wiki(session_file=SESSION_FILE):
    wiki = Wiki()
    wiki.session_file(session_file)
    return wiki

def get_secondary_wiki():
    return get_wiki(session_file=SECONDARY_SESSION_FILE)


def throttle(session_file, delay):
    wiki = get_wiki(session_file)
    if not delay:
        print(wiki.throttle)
    else:
        wiki.throttle = float(delay)
        wiki.save(session_file)

@action('throttle')
def throttle1(delay=None):
    ''' Set or get delay between queries.  '''
    throttle(SESSION_FILE, delay)

@action('throttle2')
def throttle2(delay=None):
    ''' Set or get delay between queries for secondary wiki. '''
    throttle(SECONDARY_SESSION_FILE, delay)


def read_page(session_file, *args):
    name = ' '.join(args).strip()
    if not name:
        print('Pass page name as last param')
        return
    wiki = get_wiki(session_file)
    print(wiki.page(name).read())

@action('read')
def read_page1(*args):
    ''' Print page which's name passed in param to stdout '''
    read_page(SESSION_FILE, *args)

@action('read2')
def read_page2(*args):
    ''' Print page from secondary wiki which's name passed in param to stdout '''
    read_page(SECONDARY_SESSION_FILE, *args)

def autocomplete(session_file, *args):
    prefix = ' '.join(args)
    wiki = get_wiki(session_file)
    print('\n'.join(
        wiki.opensearch(prefix)
    ))

@action('autocomplete')
def autocomplete1(*args):
    ''' Print list of page names which starts from passed prefix '''
    autocomplete(SESSION_FILE, *args)

@action('autocomplete2')
def autocomplete1(*args):
    ''' Print list of page names which starts from passed prefix for secondary wiki'''
    autocomplete(SECONDARY_SESSION_FILE, *args)

def login(session_file):
    try:
        os.remove(session_file)
    except OSError:
        pass
    get_wiki(session_file)

@action('login')
def login1():
    ''' Login to primary wiki '''
    login(SESSION_FILE)

@action('login2')
def login2():
    ''' Login to secondary wiki '''
    login(SECONDARY_SESSION_FILE)


@action('logout')
def logout():
    ''' Delete session file '''
    os.remove(SESSION_FILE)

@action('logout2')
def logout2():
    ''' Delete session file '''
    os.remove(SECONDARY_SESSION_FILE)

def write_page(session_file, *args):
    name = ' '.join(args)
    wiki = get_wiki()

    text = []
    for line in sys.stdin:
        text.append(line)

    wiki.page(name).write(''.join(text), 'cli editing')

@action('write')
def write_page1(*args):
    ''' Read text from stdio and save to page with name passed in param '''
    write_page(SESSION_FILE, *args)

@action('write2')
def write_page2(*args):
    ''' Read text from stdio and save to page on secondary wiki with name passed in param '''
    write_page(SECONDARY_SESSION_FILE, *args)

@action('now')
def get_now():
    ''' Prints current ISO-formatted zulu time '''
    print(datetime2zulu(datetime.utcnow()))



#coding=utf8

from bwikibot.cli import get_wiki, get_secondary_wiki, action


@action('bulk_export')
def bulk_export(*title_prefix):
    ''' Copy pages with given prefix from primary wiki to secondary. '''
    source_wiki = get_wiki()
    dest_wiki = get_secondary_wiki()

    comment = 'Автоматичний експорт з {}'.format(
        source_wiki.endpoint.split('/')[2]
    )

    for page in source_wiki.get_pages(prefix=' '.join(title_prefix)):
        print('Exporting page: {}'.format(page.title))
        dest_page = dest_wiki.page(page.title)
        if not dest_page.exists():
            dest_page.write(page.read(), comment)
        else:
            print('\tPage already exists!')

def bulk_delete(wiki_chooser, title_prefix, reason):
    reason = reason or 'Автоматичне видалення всіх з префіксом "{}"'.format(
        title_prefix
    )
    wiki = wiki_chooser()

    for page in wiki.get_pages(prefix=title_prefix):
        print('Deleting page: {}'.format(page.title))
        page.delete(reason)

@action('bulk_delete2')
def bulk_delete2(title_prefix, reason=None):
    ''' Delete all pages of secondary wiki which starts with prefix '''
    bulk_delete(get_secondary_wiki, title_prefix, reason)

@action('bulk_delete')
def bulk_delete1(title_prefix, reason=None):
    ''' Delete all pages of primary wiki which starts with prefix '''
    bulk_delete(get_wiki, title_prefix, reason)



import re
from datetime import datetime

months = [
    'нульня',
    'січня',
    'лютого',
    'березня',
    'квітня',
    'травня',
    'липня',
    'серпня',
    'вересня',
    'жовтня',
    'листопада',
    'листопада',
    'грудня'
]

months_nums = {name: num for num, name in enumerate(months)}

def parse_signature_time(timestamp):
    ''' Returns datetime
        >>> parse_signature_time('23:54, 13 листопада 2012 (UTC)')
        datetime.datetime(2012, 11, 13, 23, 54)
    '''
    match = re.match('(\d\d):(\d\d), (\d+) (\w+) (\d{4}) \(UTC\)', timestamp)
    if not match:
        return
    groups = match.groups()

    return datetime(
        hour=int(groups[0]),
        minute=int(groups[1]),
        day=int(groups[2]),
        month=months_nums[groups[3]],
        year=int(groups[4])
    )


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#coding=utf8
from datetime import timedelta, datetime
import sys

from mwparserfromhell import parse as parse_markup

from bwikibot.cli import get_wiki, action
from bwikibot.api import datetime2zulu, zulu2datetime, WikiError
from bwikibot.ui import cprint

@action('check_uploads')
def main(start_from, limit):
    ''' Check and mark new files for licensing issues,
        and send messages to uploaders.'''
    wiki = get_wiki()
    if start_from in ('old', 'new'):
        start_from = get_bot_param(wiki, 'BunykBot', start_from + ' start')
    UploadChecker(wiki, start_from, limit)

class UploadChecker:
    def __init__(self, wiki, start_from, limit):
        self.limit = int(limit)
        self.wiki = wiki
        self.marked_counter = 0
        for upload in self.get_uploads(start_from):
            self.check_upload(upload)

    def get_uploads(self, start):
        start = zulu2datetime(start) if start else None
        for upload in self.wiki.logevents(
                event_type='upload', start=start,
                direction='newer',
        ):
            yield upload

    def check_upload(self, upload):
        if upload.time + wait_before_check > datetime.utcnow():
            return
        print('Checking image {} uploaded at {} by {}'.format(
            upload.page.title,
            datetime2zulu(upload.time),
            upload.user.name,
        ))
        if not upload.page.exists():
            print('already deleted')
            return
        redirect = upload.page.redirect()
        if redirect:
            print('Redirect to:', redirect.title)
            return
        diagnosis = self.diagnose(upload.page)
        cprint('diagnosis:', diagnosis, color='red')
        return 
        if diagnosis:
            summary = license_summaries[diagnosis]
            warn(upload.user, upload.page.title, summary['summary'])
            mark_image(upload.page, summary) 
            self.marked_counter += 1
            if self.marked_counter >= self.limit:
                save_and_exit(datetime2zulu(upload.time))
        else:
            print('ok')

    def diagnose(self, page_to_check):
        '''
            False - ok
            'fair_use': 'Потребує обгрунтування добропорядного користування',
            'author': 'Для нього зазначений автор і джерело, однак нема підтвердження того, що автор цього твору дозволяє ліцензувати цей файл на зазначених умовах'
            'untagged': 'Відсутня правова інформація',
            'no_template': 'Нема шаблону ліцензії',
            'no_source': 'Не зазначене джерело',
        '''
        wikicode = parse_markup(page_to_check.read())
        image_template = None
        fair_use_template = None
        need_source = False
        not_need_source = False
        for template in wikicode.filter_templates(recursive=True):
            t_name = str(template.name).strip()
            t_page = self.wiki.page('Шаблон:' + t_name).resolve_redirect()
            t_name = t_page.title.split(':')[1]
            print(t_name)
            if t_name == 'Зображення': 
                image_template = template
            elif t_name == 'Обґрунтування добропорядного використання':
                fair_use_template = template
            else:
                cprint(t_page.title, color='green')
                for cat in t_page.categories():
                    ct = str(cat.title)
                    print('\t', ct)
                    if ct == warnings_cat:
                        return False
                    if ct == need_source_cat:
                        need_source = True
                    if ct == not_need_source_cat:
                        not_need_source = True

        if not image_template:
            return 'untagged'
        else:
            if need_source:
                source = image_template.get('Джерело').value
                source = fake.sub('', str(source)).strip()
                if source:
                    return False
                else:
                    return 'no_source'
            else:
                if not_need_source:
                    return False
                else:
                    return 'no_template'

last_read_value = None
def get_bot_param(wiki, bot_name, name):
    global last_read_value
    last_read_value = (wiki, bot_name, name)
    res = wiki.page('%s:%s/%s' % (
        wiki.USER_NS_NAME, bot_name, name
    )).read()
    print('%s/%s = %s' % (bot_name, name, res))
    return res

def set_bot_param(wiki, bot_name, name, value):
    wiki.page('%s:%s/%s' % (
        wiki.USER_NS_NAME, bot_name, name
    )).write(value, 'Записуємо зроблене')

def save_and_exit(value):
    if last_read_value:
        set_bot_param(*(last_read_value + (value, )))
    sys.exit(0)


wait_before_check = timedelta(hours=1, minutes=30)

user_welcome = '{{subst:welcome}}--~~~~\n\n'
problem_images_tag = '<!-- problem images list -->'

user_warning = '''
{{subst:Проблемні зображення}}
%%(images)s
%(tag)s
--~~~~
''' % {'tag': problem_images_tag}

image_issue = '* [[:%(image)s|%(image)s]]: %(summary)s\n'
usertalk_summary = 'Робот: попередження про проблеми з ліцензуванням зображень'
license_summaries= {
    'untagged': {
        'image': '{{subst:nld}}',
        'summary': 'Відсутня правова інформація',
    },
    'no_license': {
        'image': '{{subst:nld}}',
        'summary': 'Нема шаблону ліцензії',
    },
    'no_source': {
        'image': '{{subst:nsd}}',
        'summary': 'Не зазначене джерело',
    },
    'prohibited': {
        'image': '{{subst:nld}}',
        'summary': 'Використана заборонена ліцензія',
    },
    'old': {
        'image': '{{subst:nld}}',
        'summary': 'Використана застаріла ліцензія',
    },
    'no_template': {
        'image': '{{subst:nld}}',
        'summary': 'Нема шаблону ліцензії',
    },
}

category_by_date = "Файли з нез'ясованим статусом від %(day)d %(month)s %(year)d"
month_names = [0, 'січня', 'лютого', 'березня', 'квітня', 'травня', 'червня', 'липня', 'серпня', 'вересня', 'жовтня', 'листопада', 'грудня']
category_content = "[[Категорія:Файли з нез'ясованим статусом|%(month)d-%(day)02d]]\n"
category_summary = 'Робот: автоматичне створення категорії'


def mark_image(page, summary):
    check_category(page.wiki)
    try:
        page.write(
            (page.read() or '') + summary['image'],
            summary['summary']
        )
    except WikiError as e:
        print('Cannot mark image because of', e)

def check_category(wiki):
    if check_category.created:
        return
    now = datetime.utcnow()
    name = category_by_date % {
        'day': now.day,
        'month': month_names[now.month],
        'year': now.year
    }
    cat = wiki.category(name)
    if not cat.exists():
        cat.write(
            category_content % {
                'day': now.day,
                'month': now.month,
            },
            category_summary
        )
        print('Created', cat)
    check_category.created = True
check_category.created = False

def warn(user, image, problem):
    images = image_issue % {'image': image, 'summary': problem}
    talkpage = user.talkpage()
    talktext = talkpage.read() or user_welcome
    # check if user was warned
    pos = talktext.rfind(problem_images_tag)
    if pos >= 0:
        # check if there was new topics in talk
        pos2 = talktext.rfind('=', pos)
        if pos2 >= 0:
            # if where was - add full message to the end
            talktext += user_warning % {'images': images}
        else:
            # add new lines to old messages
            talktext = talktext[:pos] + images + talktext[pos:]
    else:
        # first warning
        talktext += user_warning % {'images': images}
    try:
        talkpage.write(talktext, usertalk_summary)
        print('User warned: ' + user.name)
    except Exception as e:
        print('User {} not warned because of {}'.format(user.name, e))

warnings_cat = 'Категорія:Шаблони повідомлень про проблеми з завантаженням'
need_source_cat = 'Категорія:Шаблони ліцензій, що потребують джерела'
not_need_source_cat = 'Категорія:Шаблони ліцензій з необов’язковим джерелом'




from datetime import datetime, timedelta
from time import sleep
from subprocess import Popen

import pymongo

PORT = 31415

def get_mongo_server():
    return Popen(['mongod', '--quiet', '--port', str(PORT), '--dbpath', 'mongo_data/'])

class Cache:
    def __init__(self, name):
        self.server = get_mongo_server()
        sleep(10) # wait for server to start
        connection = pymongo.MongoClient('localhost', PORT)
        self.collection = connection[name]['cache']

        self.collection.ensure_index(
            [('key', pymongo.DESCENDING)]
        )
        self.collection.ensure_index(
            [('ts', pymongo.DESCENDING)], 
            # expireAfterSeconds=10 # not working
        )
    def __del__(self):
        self.server.terminate()

    def set(self, key, value):
        self.collection.insert({
            'key': key,
            'ts': datetime.now(),
            'value': value
        })

    def get(self, key):
        res = self.collection.find_one({'key': key})
        if datetime.now() - res['ts'] > timedelta(seconds=20):

            return None
        



#coding=utf-8
'''
User interface utilities
'''
#from mako.template import Template

colors = {
    '-': 91,
    '+': 92,
    '?': 94,
    'red': 91,
    'green': 92,
    'blue': 94,
}

def cprint(*args, **kwargs):
    color = colors.get(kwargs.get('color'), 0)
    if color:
        print('\033[%sm' % color, *(args + ('\033[0m',)))
    else:
        print(*args)

def print_diff(text_a, text_b):
    ''' Print difference between two values '''
    a_lines = text_a.splitlines(1)
    b_lines = text_b.splitlines(1)
    diff = ndiff(a_lines, b_lines)

    for line in diff:
        cprint(line[:-1], color=line[0])

def ask_y_n(question):
    while True:
        ans = input(question)
        if ans.lower() == 'y':
            return True
        if ans.lower() == 'n':
            return False
        if not ans.strip():
            return ''
        print('дозволені тільки відповіді y/n та порожня')


def shift_block(block, spaces):
    ''' shifts lines of block of text to right '''
    lines = block.splitlines()
    lines = [' ' * spaces + line.strip() for line in lines]
    return '\n'.join(lines).strip()

def link(text):
    print(text)
    if isinstance(text, tuple):
        text = '|'.join(text)
    return '[[{}]]'.format(text) if text else text

def link_list(items, separator=', '):
    if isinstance(items, str):
        items = eval(items)
    return separator.join(map(link, items))

def render(template, **kwargs):
    return Template(template).render(
        link=link,
        link_list=link_list,
        **kwargs
    )



import argparse
from pprint import pprint
import sys

from bwikibot.api import Wiki
from bwikibot import lexer
from bwikibot.cli import action

parser = argparse.ArgumentParser(
    description='Get some text and print another'
)

parser.add_argument(
    '-f', dest='from_lang', default="en",
    help='language to translate from'
)
parser.add_argument(
    '-t', dest='to_lang', default="uk",
    help='language to translate to'
)

traslate_page_name = None

@action('translate')
def main(*argv):
    ''' Automate translation of wikitext from one language to another
        -f from_lang (for example 'en')
        -t to lang (for example 'uk')
    '''
    global traslate_page_name
    args = parser.parse_args(argv)

    wiki = Wiki('http://%s.wikipedia.org/w/api.php' % args.from_lang)
    traslate_page_name = get_translate_page_name(wiki, args)
    for line in sys.stdin:
        if line.strip():
            print(translate(line[:-1]))
        else:
            print()

def get_translate_page_name(wiki, args):
    def traslate_page_name(name):
        page = wiki.page(name)
        for i in range(5): # follow 5 redirects max
            redirect = page.redirect()
            if not redirect:
                break
            else:
                page = redirect
        return page.interwikis().get(args.to_lang, name)
    return traslate_page_name

def save_case(text, translation):
    if text[0].islower():
        return translation[0].lower() + translation[1:]
    else:
        return translation

def translate_segment(text):
    res = save_case(text, traslate_page_name(text))
    return res

def translate(text):
    tokens = lexer.tokenize(text)
    res = []
    for token in tokens:
        if token[0] == lexer.LINK:
            trans = translate_segment(token[1])
            full = token[1] == token[2] and ':' in token[1]
            token = token[0], trans, trans if full else token[2]
        res.append(lexer.token_text(token))
    return ''.join(res)



""" Mediawiki markup lexer """

import re

TEXT, COMMENT, LINK, MARKUP, VERBATIM, HEADER, INTERWIKI = range(7)

def text(scaner, token):
    return (TEXT, token)

def link(scaner, token):
    a = re.match(r"\[\[([^\|\]]+)(\|[^\]\|]+)?\]\]",token)
    assert a != None , 'scanner get a bad link: ' + token
    page = text = ""
    if len(a.groups()) == 1:
        page = text = a.groups()[0]
    if len(a.groups()) == 2:
        page, text = a.groups()
        if text == None:
            text = page
        else:
            text = text[1:]
    return (LINK, page, text)
    
def comment(scaner, token):
    return (COMMENT, token)

def markup(scaner, token):
    return (MARKUP, token)

def verbatim(scaner, token):
    return (VERBATIM, token)

def header(scaner, token):
    a = re.match(r"\n(=+)(.*)(\1)\n",token)
    level = len(a.groups()[0])
    return (HEADER,a.groups()[1],level)

def interwiki(scaner,token):
    a = re.match(r"\[\[([^:]*):([^\]]*)\]\]",token)
    return (INTERWIKI,a.groups()[0],a.groups()[1])

def token_text(token): # translate token tuple back to text
    if token[0] == LINK:
        res = '[[' + token[1]
        if token[1] != token[2]:
            res += '|' + token[2]
        res += ']]'
        return res
    elif token[0] == HEADER:
        return '\n' + "=" * token[2] + token[1] + "=" * token[2] + '\n'
    elif token[0] == INTERWIKI:
        return '[[' + token[1] + ':' + token[2] + ']]'                
    else:
        return token[1]

iwiki_regex = r"\[\[(en|de|fr|zh|nl|pl|it|ja|es|pt|ru|sv|lt|no|fi|ca|uk|hu|cs|tr|ro|vo|eo|da|ko|id|sk|ar|vi|sr|he|bg|fa|sl|lmo|et|hr|new|te|nn|th|gl|el|ceb|simple|ms|eu|ht|bs|bpy|lb|ka|is|sq|la|br|hi|az|bn|mk|mr|sh|tl|cy|io|pms|lv|ta|su|oc|jv|nap|nds|scn|be|ast|ku|wa|af|be-x-old|an|ksh|szl|fy|frr|zh-yue|ur|ia|ga|yi|sw|als|hy|am|roa-rup|map-bms|bh|co|cv|dv|nds-nl|fo|fur|glk|gu|ilo|kn|pam|csb|kk|km|lij|li|ml|gv|mi|mt|nah|ne|nrm|se|nov|qu|os|pi|pag|ps|pdc|rm|bat-smg|sa|gd|sco|sc|si|tg|roa-tara|tt|to|tk|hsb|uz|vec|fiu-vro|wuu|war|vls|yo|diq|zh-min-nan|zh-classical|frp|lad|bar|bcl|kw|mn|haw|ang|ln|ie|wo|tpi|ty|crh|jbo|ay|zea|eml|ky|ig|or|mg|cbk-zam|kg|arc|rmy|gn|mo|so|kab|ks|stq|ce|udm|mzn|pap|cu|sah|tet|sd|lo|ba|pnb|iu|na|got|bo|dsb|chr|cdo|hak|om|my|sm|ee|pcd|ug|as|ti|av|bm|zu|pnt|nv|cr|pih|ss|ve|bi|rw|ch|arz|xh|kl|ik|bug|dz|ts|tn|kv|tum|xal|st|tw|bxr|ak|ab|ny|fj|lbe|ki|za|ff|lg|sn|ha|sg|ii|cho|rn|mh|chy|ng|kj|ho|mus|kr|hz|mwl|pa):([^\]]*)\]\]"
redirect_regex = r"#\w+ \[\[([^\]]+)\]\]"

def redirect_to(text):
    ''' Where redirect redirects '''
    a = re.findall(redirect_regex,text)
    if a:
        return a[0]
    else:
        return None

lexer = re.Scanner([
    (r"<!--(.|\n)*?-->", comment),
    (r"<(source|syntaxhighlight|math|nowiki).*?>(.|\n)*?</(\1)>", verbatim),
    (iwiki_regex,interwiki), #interwikis
    (r"\[\[(File|Image):[^|\]]+\.(jpg|png|svg|gif)\|([a-z0-9]+\|)*",markup), #images, files
    (r"\[\[[^\|\]]+(\|[^\]\|]+){2,}\]\]",markup),#files in other languages
    (r"\[\[[^\|\]]+(\|[^\]\|]+)?\]\]",link), # internal links
    (r"\[[^\]]+\]",markup), # external links
    (r"(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?",markup), #external link too
    (r"\n(=+)([^\n]*)(\1)\n",header),
    (r"\|\}\}",markup),
    (r"\{\|",markup),
    (r"\|\}",markup),
    (r"\{\{",markup),
    (r"\}\}",markup),
    (r"\{",markup), # !!!
    (r"\}",markup), # !!!
    (r"\]\]",markup),
    ("<[\"\s<]",text),
    (r"<[a-zA-Z/][^>]*>", markup),
    (r"'''",markup), 
    (r"''",markup), 
    (r"\n[*#]+:*",markup),
    (r"\n:+",markup),
    (r"\n\|-",markup),
    (r"\|",markup),
    (r"\n!",markup),
    (r"==+",markup),
    (r"=",markup),
    (r":",markup),
    (r"\s",text),
    (r"\[\]",text),#i quite surprised but it really is text
    (r"((?!''+)[^\|\{\}\[\]:=<\n])*",text),
    (r"<\s",text),
    (r"[^<]*",text)
    ])

class ParseError(Exception):
    pass

def tokenize(text):
    res = lexer.scan(text)
    if res[1]:
        raise ParseError('Markup is not fully parsed')
    return res[0]


def clean_markup(markup):
    lex,rest = parser(markup)
    res = [" "]
    for i in lex:
        if i[0]==LINK:
            res.append(i[2])
        elif i[0]==VERBATIM:
            res.append(i[1])
        elif i[0]==HEADER:
            res.append(i[1])
        elif i[0]==TEXT:
            res.append(i[1])
        else:
            if res[-1]!=" ":
                res.append(" ")
    return "".join(res)



#coding=utf-8
from collections import defaultdict
import os
import sys

from mwparserfromhell import parse as parse_markup

from bwikibot.cli import get_wiki, get_secondary_wiki, action
from bwikibot.ui import render

class ImportException(Exception):
    def __init__(self, msg):
        self.msg = msg

@action('import_poster')
def import_poster(film_name, source='en'):
    ''' Імпортувати постер для статті, і додати його в шаблон фільму.
        Перший параметр - назва статті фільму,
        другий - 
    '''
    wiki = get_wiki()
    try:
        image_name, author, source = download_poster_to_tmp(wiki, source, film_name)
        upload_poster_to_dest_wiki(wiki, film_name, source, author, image_name)
        add_poster_to_infobox(
            wiki.page(film_name).resolve_redirect(),
            image_name
        )
        os.remove('tmp')
    except ImportException as e:
        print(e.msg)

def download_poster_to_tmp(wiki, source, film_name):
    image_name, author = None, None
    if source in ('en', 'ru', 'de', 'fr'):
        sfilm_name = (wiki
            .page(film_name)
            .resolve_redirect()
            .interwikis().get(source)
        )
        if not sfilm_name:
            raise ImportException("Вибачте, не знайдено інтервікі до %s" % film_name)

        swiki = wiki.get_unified_login_to('%s.wikipedia.org' % source)
        image_name, author = download_poster_from_source_wiki(swiki, sfilm_name)
        source = 'http://{lang}.wikipedia.org/wiki/File:{image_name}'.format(
            lang=source, image_name = image_name
        )
    else:
        try:
            headers, content = wiki.http.request(source)
        except:
            raise ImportException("Не можу завантажити %s" % source)
        with open('tmp', 'wb') as f:
            f.write(content)
    image_name = image_name or input('Назва файлу для картинки:')
    author = author or input('Автор:')
    return image_name, author, source

def download_poster_from_source_wiki(swiki, sfilm_name):
    sfp = swiki.page(sfilm_name).resolve_redirect()
    image = None
    for template in parse_markup(sfp.read()).filter_templates():
        tn = str(template.name).strip().lower().replace('_', ' ')
        if (tn.startswith('infobox') and tn.endswith('film')):
            image = template.get('image')
            distributor = template.get('distributor')
    image_name = str(image.value).strip() if image else ''
    if not image_name:
        raise ImportException('Зображення не знайдене на сторінці %s' % sfilm_name)

    image = swiki.file(image_name)
    image.download('tmp')

    distributor = str(distributor.value).strip() if distributor else ''
    return image_name, distributor

def upload_poster_to_dest_wiki(wiki, film_name, source, author, image_name):
    dfp = wiki.page(film_name)
    fp = wiki.file(image_name)
    if fp.exists():
        raise ImportException('Зображення "%s" вже існує' % image_name)

    description = '''{{{{Зображення
|Назва={film_name}
|Опис=Постер фільму "[[{film_name}]]"
|Автор={author}
|Джерело={source}
|Час створення=
|Ліцензія=нижче
}}}}
{{{{subst:Постер з ОДВ|{film_name}}}}}'''.format(
        film_name=film_name,
        author=author,
        source=source
    )
    fp.upload('tmp', 'Автоматичний імпорт постера', description)

def add_poster_to_infobox(page, poster):
    include = '[[Файл:%s|200px]]' % poster
    markup = parse_markup(page.read())
    for t in markup.filter_templates():
        if str(t.name).strip().lower() == 'фільм':
            t.add('плакат',
                include,  
                force_nonconformity=True
            )
    page.write(str(markup), 'Додав постер фільму')


template = r"""
{{Фільм
| українська назва  = ${title_uk}
| оригінальна назва = ${title}
| плакат            = [[${poster_file}|200px]]
| режисер           = ${director|link}
| продюсер          = ${producer|link}
| сценарист         = ${written_by|link}
| актори            = ${actors|link_list}
| кінокомпанія      = ${distributors|link_list}
| країна            = ${countries|link_list}
| рік               = ${year}
| мова              = ${languages|link_list}
| тривалість        = ${running_minutes} хв.
| кошторис          = ${budget}
| касові збори      = ${box_office}
| ідентифікатор     = ${imdb_id}
}}
'''${title_uk}''' ({{lang-${lang_code}|${title}}}) - фільм.

== В ролях ==
% for actor in actors:
* ${actor|link}
% endfor

== Посилання ==
% if imdb_id:
* {{Imdb title|${imdb_id}|${title}}} 
% endif

${link_list(categories, separator='\n')}

${interwikis}
"""

info = defaultdict(lambda :'',
    title_uk='Хмарний Атлас',
    title='Cloud Atlas',
    poster_file='Файл:example.jpg',
    director='director',
    producer='producer',
    written_by='сценарист',
    actors=['Том Хенкс', 'Холлі Бері'],
    distributors=['Warner brothers'],
    countries=['США'],
    year=2012,
    languages=[('Англійська мова', 'Англійська')],
    running_minutes=172,
    budget='102000000',
    box_office='102 млн.',
    imdb_id=1371111,
    lang_code='en',
    categories=['Категорія:Фільми {}'.format(2012),],
    interwikis='[[en:Cloud Atlas]]',
)

if __name__ == '__main__':
    print(render(template, **info))
